;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-

;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          conditions.lisp
;;;; Purpose:       cl-couchdb conditions.
;;;; Programmer:    Nicolas Lamirault <nicolas.lamirault@gmail.com>
;;;;
;;;; This file, part of cl-couchdb, is Copyright (c) 2007 by Nicolas Lamirault
;;;;
;;;; cl-couchdb users are granted the rights to distribute and use this software
;;;; as governed by the terms of the Lisp Lesser GNU Public License
;;;; (http://opensource.franz.com/preamble.html), also known as the LLGPL.
;;;;
;;;; *************************************************************************


(in-package :cl-couchdb)



(define-condition cl-couchdb-error (simple-error)
  ()
  (:documentation "Main Cl-Couchdb error."))


(define-condition couch-connection-error (cl-couchdb-error)
  ((host :reader host
         :initarg :host
         :documentation "The CouchDB server")
   (port :reader port
         :initarg :port
         :documentation "Port of the CouchDB server."))
  (:documentation "This error is signaled when the connection to the
CouchDB server failed.")
  (:report (lambda (condition stream)
             (format stream "Connection to CouchDB server : host ~A, port ~A failed"
                     (host condition) (port condition)))))


(define-condition couch-request-error (cl-couchdb-error)
  ((code :reader code
         :initarg :code
         :documentation "The error code.")
   (message :reader message
            :initarg :message
            :documentation "Explanation message."))
  (:documentation "CouchDB server request error.")
  (:report (lambda (condition stream)
             (format stream "CouchDB server error ~A : ~A."
                     (code condition) (message condition)))))


(define-condition couch-database-already-exists-error (couch-request-error)
  ()
  (:documentation "This error is signaled when the client try to
creates a database which already exists.")
  (:report (lambda (condition stream)
             (format stream "CouchDB database already exists : ~A."
                     (message condition)))))


(define-condition couch-database-doesnt-exists-error (couch-request-error)
  ()
  (:documentation "This error is signaled when the client try to
deletes a database which doens't exists.")
  (:report (lambda (condition stream)
             (format stream "CouchDB database doesn't exists : ~A."
                     (message condition)))))


;; (define-condition couch-unknown-view-error (couch-request-error)
;;   ()
;;   (:documentation "This error is signaled when the client try to
;; view a document specifying an unknown view.")
;;   (:report (lambda (condition stream)
;;              (format stream "CouchDB unknown view : ~A."
;;                      (message condition)))))

