;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-

;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          db.lisp
;;;; Purpose:       Unit Tests for the Database Protocol.
;;;; Programmer:    Nicolas Lamirault <nicolas.lamirault@gmail.com>
;;;;
;;;; This file, part of cl-couchdb, is Copyright (c) 2007 by Nicolas Lamirault
;;;;
;;;; cl-couchdb busers are granted the rights to distribute and use this software
;;;; as governed by the terms of the Lisp Lesser GNU Public License
;;;; (http://opensource.franz.com/preamble.html), also known as the LLGPL.
;;;;
;;;; *************************************************************************


(in-package :cl-couchdb-test)


(defparameter *couch*
  (make-instance 'cl-couchdb:couch :host "localhost" :port 8888))


(lisp-unit:define-test list-db-correct
  (let ((sum 0))
    (cl-ppcre:do-matches (s e "\\["
                            (cl-couchdb:list-db *couch*))
      (incf sum))
    (lisp-unit:assert-true (> sum 0))))


(lisp-unit:define-test create-db-correct
  (let ((sum 0)
        (name (string-downcase (format nil "~A~A"
                                       (gensym)
                                       (get-universal-time)))))
    (lisp-unit:assert-equal "{\"ok\":true}"
                            (cl-couchdb:create-db *couch* name))))


(lisp-unit:define-test create-db-incorrect
  (let ((name (string-downcase (format nil "~A~A"
                                       (gensym) (get-universal-time)))))
    (cl-couchdb:create-db *couch* name)
    (lisp-unit:assert-error
     'cl-couchdb:couch-database-already-exists-error
     (cl-couchdb:create-db *couch* name))))


(lisp-unit:define-test create-db-incorrect-upcase-name
  (let ((name (string-upcase (format nil "~A~A"
                                     (gensym)  (get-universal-time)))))
    (lisp-unit:assert-error
     'cl-couchdb:couch-request-error
     (cl-couchdb:create-db *couch* name))))


(lisp-unit:define-test create-db-incorrect-digit-name
  (let ((name (string-upcase (format nil "~A"
                                     (get-universal-time)))))
    (lisp-unit:assert-error
     'cl-couchdb:couch-request-error
     (cl-couchdb:create-db *couch* name))))


(lisp-unit:define-test info-db-correct
  (let ((sum 0)
        (name (string-downcase (format nil "~A~A"
                                       (gensym) (get-universal-time)))))
    (cl-couchdb:create-db *couch* name)
    (cl-ppcre:do-matches (s e "db_name"
                            (cl-couchdb:info-db *couch* name))
      (incf sum))
    (lisp-unit:assert-true (> sum 0))))


(lisp-unit:define-test info-db-incorrect
  (let ((name (string-downcase (format nil "~A~A"
                                       (gensym) (get-universal-time)))))
    (lisp-unit:assert-error
     'cl-couchdb:couch-database-doesnt-exists-error
     (cl-couchdb:info-db *couch* name))))


(lisp-unit:define-test delete-db-correct
  (let ((sum 0)
        (name (string-downcase (format nil "~A~A"
                                       (gensym) (get-universal-time)))))
    (cl-couchdb:create-db *couch* name)
    (lisp-unit:assert-equal "{\"ok\":true}"
                            (cl-couchdb:delete-db *couch* name))))


(lisp-unit:define-test delete-db-incorrect
  (let ((name (string-downcase (format nil "~A~A"
                                       (gensym) (get-universal-time)))))
    (lisp-unit:assert-error
     'cl-couchdb:couch-database-doesnt-exists-error
     (cl-couchdb:delete-db *couch* name))))


