;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-

;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          doc.lisp
;;;; Purpose:       Unit Tests for the Document Protocol.
;;;; Programmer:    Nicolas Lamirault <nicolas.lamirault@gmail.com>
;;;;
;;;; This file, part of cl-couchdb, is Copyright (c) 2007 by Nicolas Lamirault
;;;;
;;;; cl-couchdb users are granted the rights to distribute and use this software
;;;; as governed by the terms of the Lisp Lesser GNU Public License
;;;; (http://opensource.franz.com/preamble.html), also known as the LLGPL.
;;;;
;;;; *************************************************************************


(in-package :cl-couchdb-test)


(defparameter *couch*
  (make-instance 'cl-couchdb:couch :host "localhost" :port 8888))


(defparameter *db-test-name* (string-downcase (format nil "~A" (gensym))))


(defmacro with-couch-db (couch name &body body)
  `(handler-case 
       (progn
         (cl-couchdb:info-db ,couch ,name)
         ,@body)
     (cl-couchdb:couch-database-doesnt-exists-error (condition)
       (cl-couchdb:create-db ,couch ,name)
       ,@body)))


(lisp-unit:define-test list-document-correct
  (with-couch-db *couch* *db-test-name*
    (let ((sum 0))
      (cl-ppcre:do-matches (s e "view"
                              (cl-couchdb:list-documents *couch* *db-test-name*))
        (incf sum))
      (lisp-unit:assert-true (= sum 1)))))


(lisp-unit:define-test list-document-incorrect
  (let ((name (string-downcase
               (format nil "~A~A" (gensym) (get-universal-time)))))
    (lisp-unit:assert-error
     'cl-couchdb:couch-database-doesnt-exists-error
     (cl-couchdb:list-documents *couch* name))))
   

(lisp-unit:define-test create-document-correct
  (let ((content
         (json:encode-json-alist-to-string '(("author" "nicolas")))))
    (with-couch-db *couch* *db-test-name*
      (let ((sum 0))
        (cl-ppcre:do-matches (s e "{\"ok\":true"
                                (cl-couchdb:save-document *couch*
                                                          *db-test-name*
                                                          content))
          (incf sum))
        (lisp-unit:assert-true (= sum 1))))))


(lisp-unit:define-test create-document-incorrect
  (let ((name (string-downcase
               (format nil "~A~A" (gensym) (get-universal-time))))
        (content
         (json:encode-json-alist-to-string '(("author" "nicolas")))))
    (lisp-unit:assert-error
     'cl-couchdb:couch-database-doesnt-exists-error
     (cl-couchdb:save-document *couch* name content))))


(lisp-unit:define-test delete-document-correct
  (let ((content
         (json:encode-json-alist-to-string '(("author" "nicolas")))))
    (with-couch-db *couch* *db-test-name*
      (let ((data
             (cl-couchdb:save-document *couch*
                                       *db-test-name*
                                       content)))
        (lisp-unit:assert-true (not (null data)))
        (let ((res (find :_ID (json:decode-json-from-string data)
                         :key #'car)))
          (lisp-unit:assert-true (not (null (cdr res))))
          (let ((sum 0))
            (cl-ppcre:do-matches (s e "{\"ok\":true"
                                    (cl-couchdb:delete-document *couch*
                                                                *db-test-name*
                                                                (cdr res)))
              (incf sum))
            (lisp-unit:assert-true (= sum 1))))))))


(lisp-unit:define-test delete-document-incorrect
  (let ((name (string-downcase
               (format nil "~A~A" (gensym) (get-universal-time))))
        (doc-id (string-downcase
                 (format nil "~A~A" (gensym) (get-universal-time)))))
    (lisp-unit:assert-error
     'cl-couchdb:couch-database-doesnt-exists-error
     (cl-couchdb:delete-document *couch* name doc-id))))
